# RCSid:
#	$Id: cidr_subs.pl,v 1.9 2009/06/18 18:18:10 sjg Exp $
#
#	@(#)Copyright (c) 1998-2001 Simon J. Gerraty.
#	All rights reserved.
#	
#	This software is the property of Simon J. Gerraty.
#	It may be used only in accordance with the conditions outlined
#	in the accompanying LICENSE file. If your distribution does
#	not contain a LICENSE file, then you have no rights to use
#	this software and should contact sales@crufty.net to obtain a
#	licensed distribution.
#	
#	This software is provided "AS IS" in the hope that it will be
#	of use.  Since it is not possible to anticipate all possible
#	use of this software, there is absolutely NO WARRANTY.
#
#	Please forward bug reports and fixes to help@crufty.net
#


if ($0 =~ m,^(.+)/cidr_sub,) {
  push(@INC, $1);

  require 'ip_subs.pl';
  
  while (<>) {
    chop;
    @a = split(/\s+/);
    if (scalar(@a) > 1 || $_ =~ m,/,o) {
      %cidr = &CIDR(split(/\s+/, $_));
      if ($cidr{'ip2'} eq '') {
	print STDERR "$_: invalid\n";
      } else {
	printf "$cidr{'prefix'}/$cidr{'bits'}: $cidr{'ip1'} - $cidr{'ip2'}, mask: %s %#x\n", &hex2ip($cidr{'mask'}), $cidr{'mask'};
      }
      next;
    }
    printf "$_ %s a member of $cidr{'prefix'}/$cidr{'bits'}\n", (&cidr_isMember($_, %cidr)) ? "is" : "is not";
  }
}

require 'ip_subs.pl';

# valid input can be:
# ip/bits
# ip, bits
# ip1, ip2
# ip, mask
sub CIDR {
  local($ip1,$ip2) = @_;
  local(%cdir) = ();
  local($prefix) = '';
  local($bits) = '';
  local($mask) = '';
  local($a1,$a2);
  local($x,$y,$j);
  
  if ($ip2 eq '') {
    if ($ip1 =~ m,^(\d[\.\d]+)/(\d+),o) {
      $ip1 = $prefix = $1;
      $bits = $2;
    }
  } else {
    if ($ip2 =~ m/^\d+$/o) {
      $prefix = $ip1;
      $bits = $ip2;
    } elsif ($ip2 =~ m/^255\./o) {
      $mask = &ip2hex($ip2);
    } elsif ($ip2 =~ m/ff/oi) {
      $mask = hex($ip2);
    }
    if ($mask ne '') {
      $x = ~$mask;
      $y = 1;
      for ($j = 1; $j < 32; ++$j) {
	$y |= (1 << $j);
	if ($x == $y) {
	  $bits = 31 - $j;
	  last;
	}
      }
    }
  }
  if ($bits ne '') {
    if ($mask eq '') {
      $mask = ~((1<<(32 - $bits)) - 1);
    }
    $a1 = &ip2hex($ip1) & $mask;
    $cidr{'prefix'} = $cidr{'ip1'} = &hex2ip($a1);
    $cidr{'bits'} = $bits;
    $cidr{'mask'} = $mask;
    $a2 = $a1 + ~$mask;
    $cidr{'ip2'} = &hex2ip($a2);
    $cidr{'a1'} = $a1;
    $cidr{'a2'} = $a2;
  }
  %cidr;
}

# a candidate address $a is a member iff
# $a & $a1 == $a1 && $a & $a2 == $a
sub cidr_isMember {
  local($a, %cidr) = @_;
  local($a1) = $cidr{'a1'};
  local($a2) = $cidr{'a2'};
  
  if ($a =~ m/\d+\.\d+\./) {
    $a = &ip2hex($a);
  }
  ((($a & $a1) == $a1) && (($a & $a2) == $a));
}

1;
